# -*- coding: utf-8 -*-

"""
The :mod:`audiotsm.utils` module provides utility functions and classes used in
the implementation of time-scale modification procedures.

.. autoclass:: audiotsm.utils.CBuffer
    :members:
    :undoc-members:

.. autoclass:: audiotsm.utils.NormalizeBuffer
    :members:
    :undoc-members:
"""

from .cbuffer import CBuffer
from .normalizebuffer import NormalizeBuffer
