# -*- coding: utf-8 -*-

"""
audiotsm.base
~~~~~~~~~~~~~

This module provides base classes for the implementation of time-scale
modification procedures.
"""

from .tsm import TSM
from .analysis_synthesis import AnalysisSynthesisTSM, Converter
