# -*- coding: utf-8 -*-

"""
:class:`~audiotsm.base.tsm.TSM` objects use :class:`~audiotsm.io.base.Reader`
objects as input and :class:`~audiotsm.io.base.Writer` objects as output.

The :mod:`audiotsm.io` package provides Readers and Writers allowing to use
`numpy arrays`_ or `wav files`_ as input or output of a
:class:`~audiotsm.base.tsm.TSM`, to `play the output in real-time <play in
real-time>`_, as well as base classes to `implement your own <implementing your
own>`_ Readers and Writers.
"""
